/***********************************************************************/
/*                                                                     */
/* Simple sample program to get and print messages of format BALANCE.  */
/* We will read the queue until empty, then exit.                      */
/* Copyright IBM Corp 1997.                                            */
/*                                                                     */
/***********************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmqc.h>
#include "Balance.h"

#define MAX_TEXT 72

enum rc_e { good_rc, bad_rc };

int main( int argc, char *argv[] )
{

 static MQCHAR48   qmname = { '\0' };           /* Queue manager name   */
 MQHCONN           hconn;                       /* Connection handle    */
 MQHOBJ            hq;                          /* Object handle        */
 MQLONG            cc;                          /* Completion code      */
 MQLONG            rc;                          /* Reason               */
 MQLONG            msglen;                      /* Message length       */
 char              msg[ MAX_TEXT ];             /* Error Message        */
 MQOD              mqod  = { MQOD_DEFAULT };    /* Object descriptor    */
 MQMD              mqmd  = { MQMD_DEFAULT };    /* Message descriptor   */
 MQGMO             mqgmo = { MQGMO_DEFAULT };   /* Get options          */
 int               count;                       /* Messages received    */
 struct BALANCE    Balance;                     /* MQ message           */
 char              AccountHolder[ sizeof( Balance.AccountHolder ) + 1]; 
 char              AccountNumber[ sizeof( Balance.AccountNumber ) + 1]; 

 if ( argc != 2 )
 {
  sprintf( msg, "Usage: %s queue_name\n", argv[0] );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }
 strcpy( mqod.ObjectName, argv[ 1 ] );

 MQCONN( qmname, &hconn, &cc, &rc );   /* Connect to default QMgr */
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQCONN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }

 MQOPEN( hconn,
         &mqod,
         MQOO_INPUT_SHARED,
         &hq,
         &cc,
         &rc );
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQOPEN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }

 mqgmo.Options      |= MQGMO_CONVERT;
 mqgmo.WaitInterval  = MQWI_UNLIMITED;

 for ( count = 1 ; ; count++ )     /* Get until queue is empty */
 {

  memcpy( mqmd.MsgId,    MQMI_NONE, sizeof( mqmd.MsgId ) );
  memcpy( mqmd.CorrelId, MQCI_NONE, sizeof( mqmd.CorrelId ) );
 
  MQGET(  hconn,
          hq,
          &mqmd,
          &mqgmo,
          sizeof( struct BALANCE ),
         &Balance,
          &msglen,
          &cc,
          &rc );
  if ( cc != MQCC_OK )
  {
   if ( rc == MQRC_NO_MSG_AVAILABLE ) 
   {
    count--;
    break;
   }
   sprintf( msg, "MQGET failed - reason code %li\n", rc );
   fputs(   msg, stderr );
   exit(    bad_rc );
  }
  CharsToCStr( AccountHolder, Balance.AccountHolder, 
               sizeof( Balance.AccountHolder ) );
  CharsToCStr( AccountNumber, Balance.AccountNumber, 
               sizeof( Balance.AccountNumber ) );
  printf( " AccountHolder: %s\n", AccountHolder );
  printf( " AccountNumber: %s\n", AccountNumber );
  printf( " CurrentBalance: %d\n\n", Balance.CurrentBalance );

 }   /* for loop */

 printf( "\n%d messages were received - now exiting\n", count );
 MQCLOSE( hconn,
          &hq,
          MQCO_NONE,
          &cc,
          &rc );
 MQDISC( &hconn, &cc, &rc );
 exit( good_rc );

}  /* main */
