/***********************************************************************/
/*                                                                     */
/* Some string-handling utilities.                                     */
/* Copyright IBM Corp 1997.                                            */
/*                                                                     */
/***********************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "utils.h"

/***********************************************************************/
/*                                                                     */
/* Copy a null-delimited C string to output buffer of size OutSize,    */
/* not including the delimiter, and padding with blanks if required    */
/*                                                                     */
/***********************************************************************/

void CStrToChars( char *Out, char *In, int OutSize )
{

 int i;
 Boolean InDelimiterFound = False;

 for ( i = 0 ; i < OutSize ; i++ )
 {
  if ( In[ i ] == '\0' ) InDelimiterFound = True;
  if ( InDelimiterFound ) Out[ i ] = ' ';
  else Out[ i ] = In[ i ];
 }

 return;

}  /* CStrToChars */

/***********************************************************************/
/*                                                                     */
/* Copy an array of blank-padded chars into a null-delimited C string. */
/* Trailing blanks will be removed.                                    */
/* Note - Out MUST be sized to sizeof( In ) + 1 or greater.            */
/*                                                                     */
/***********************************************************************/

void CharsToCStr( char *Out, char *In, int InSize )
{

 int i;
 Boolean EndofStringFound = False;

 
 for ( i = InSize - 1, Out[0] = '\0' ; i >= 0 ; i-- )
 {
 
  if ( In[ i ] != ' ' && EndofStringFound == False )
  {
   EndofStringFound = True;
   Out[ i + 1 ] = '\0';
  }
  if ( EndofStringFound ) Out[ i ] = In[ i ];  
 }

 return;

}  /* CharsToCStr */
