/***********************************************************************/
/*                                                                     */
/* Simple sample program to put a single message of format NBALANCE.   */
/* Copyright IBM Corp 1997.                                            */
/*                                                                     */
/***********************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <cmqc.h>
#include "NBalance.h"

#define MAX_TEXT 72

enum rc_e { good_rc, bad_rc };

int main( int argc, char *argv[] )
{

 static MQCHAR48   qmname = { '\0'};            /* Queue manager name   */
 MQHCONN           hconn;                       /* Connection handle    */
 MQLONG            cc;                          /* Completion code      */
 MQLONG            rc;                          /* Reason               */
 char              msg[ MAX_TEXT ];             /* Error message        */
 int               i;                           /* Index variable       */
 static MQOD       mqod  = { MQOD_DEFAULT };    /* Object descriptor    */
 static MQMD       mqmd  = { MQMD_DEFAULT };    /* Message descriptor   */
 static MQPMO      mqpmo = { MQPMO_DEFAULT };   /* Put options          */
 int               msg_len;                     /* Message length       */
 struct NBALANCE   *NBalance;                   /* MQ msg to send       */
 int               Count;                       /* # of Balance structs */

 if ( argc !=2 )
 {
  sprintf( msg, "Usage: %s queue_name\n", argv[0] );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }
 strcpy( mqod.ObjectName, argv[ 1 ] );

 MQCONN( qmname, &hconn, &cc, &rc );  /* Connect to default QMgr */
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQCONN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }
 
 strncpy( mqmd.Format, "NBALANCE", sizeof( mqmd.Format ) );
 Count = 2;         /* Number of BALANCE structures to send */
 msg_len = sizeof( int ) + Count * sizeof( struct BALANCE );
 NBalance = calloc( msg_len, sizeof( MQBYTE ) );
 if ( NBalance == NULL )
 {
  fprintf( stderr, "calloc failed\n" );
  exit(    bad_rc );
 }

 NBalance -> Count = Count;
 CStrToChars( NBalance -> Balance[0].AccountHolder, "Megabucks Stefan", 
              sizeof( NBalance -> Balance[0].AccountHolder ) );
 CStrToChars( NBalance -> Balance[0].AccountNumber, "123456", 
              sizeof( NBalance -> Balance[0].AccountNumber ) );
 NBalance -> Balance[0].CurrentBalance = 3000000;
 CStrToChars( NBalance -> Balance[1].AccountHolder, "Penury Stefan", 
              sizeof( NBalance -> Balance[1].AccountHolder ) );
 CStrToChars( NBalance -> Balance[1].AccountNumber, "123321", 
              sizeof( NBalance -> Balance[1].AccountNumber ) );
 NBalance -> Balance[1].CurrentBalance = 2;
 
 MQPUT1( hconn,
        &mqod,
        &mqmd,
        &mqpmo,
         msg_len,
         NBalance,
        &cc,
        &rc );
  if ( cc != MQCC_OK )
  {
   sprintf( msg, "MQPUT1 failed - reason code %li\n", rc );
   fputs(   msg, stderr );
   exit(    bad_rc );
  }

  MQDISC( &hconn, &cc, &rc );
  exit( good_rc );

}  /* main */
