/***********************************************************************/
/*                                                                     */
/* Simple sample program to get and print messages of type mqmsg.   */
/* In this example, messages are in PCF format                         */
/* We will read the queue until empty, then exit.                      */
/* Copyright IBM Corp 1997.                                            */
/*                                                                     */
/***********************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmqc.h>
#include <cmqcfc.h>
#include "NBalance.h"

#define MAX_TEXT 72

enum rc_e { good_rc, bad_rc };

int main( int argc, char *argv[] )
{

 static MQCHAR48   qmname = { '\0' };           /* Queue manager name   */
 MQHCONN           hconn;                       /* Connection handle    */
 MQHOBJ            hq;                          /* Object handle        */
 MQLONG            cc;                          /* Completion code      */
 MQLONG            rc;                          /* Reason               */
 MQLONG            msglen;                      /* Message length       */
 MQLONG            buflen;                      /* Buffer length        */
 char              msg[ MAX_TEXT ];             /* Error Message        */
 MQOD              mqod  = { MQOD_DEFAULT };    /* Object descriptor    */
 MQMD              mqmd  = { MQMD_DEFAULT };    /* Message descriptor   */
 MQGMO             mqgmo = { MQGMO_DEFAULT };   /* Get options          */
 int               MsgNo;                       /* Messages received    */
 MQCFH            *mqmsg;                       /* MQ message           */
 int               i;                           /* Counter              */
 int               Count;                       /* Number of balances   */
 MQCFIL            *Balances;                   /* Balances             */
 MQCFSL            *AccountHolders;             /* Account Holders      */
 MQCFSL            *AccountNumbers;             /* Account Numbers      */
 MQCFIL            *Ptr;                        /* Start of substructure*/
 MQBYTE            *bPtr;                       /* Ptr as byte ptr      */
 PMQCHAR           AccountHolderStr;            /* Working ptr          */
 PMQCHAR           AccountNumberStr;            /* Working ptr          */
 int               ParameterCount;              /* # sub-structures     */
 char              AccountHolder[ ACCOUNTHOLDERLEN + 1];  /* As C string*/ 
 char              AccountNumber[ ACCOUNTNUMBERLEN + 1];  /* As C string*/

 if ( argc != 2 )
 {
  sprintf( msg, "Usage: %s queue_name\n", argv[0] );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }
 strcpy( mqod.ObjectName, argv[ 1 ] );

 MQCONN( qmname, &hconn, &cc, &rc );   /* Connect to default QMgr */
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQCONN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }

 MQOPEN( hconn,
         &mqod,
         MQOO_INPUT_SHARED,
         &hq,
         &cc,
         &rc );
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQOPEN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }

 /* Allocate storage for input message                              */

 buflen =    MQCFH_STRUC_LENGTH + MQCFIL_STRUC_LENGTH_FIXED + 
           ( 2 * MQCFSL_STRUC_LENGTH_FIXED ) +
           ( MAXBALANCES * ( sizeof( MQLONG ) + ( sizeof( MQCHAR ) * 
           ( ACCOUNTHOLDERLEN + ACCOUNTNUMBERLEN ) ) ) ); 
 mqmsg = calloc( buflen, sizeof( MQBYTE ) );
 if ( mqmsg == NULL )
 {
  fprintf( stderr, "calloc failed\n" );
  exit(    bad_rc );
 }

 mqgmo.Options      |= MQGMO_CONVERT;
 mqgmo.WaitInterval  = MQWI_UNLIMITED;

 for ( MsgNo = 1 ; ; MsgNo++ )     /* Get until queue is empty */
 {

  memcpy( mqmd.MsgId,    MQMI_NONE, sizeof( mqmd.MsgId ) );
  memcpy( mqmd.CorrelId, MQCI_NONE, sizeof( mqmd.CorrelId ) );

  MQGET(  hconn,
          hq,
          &mqmd,
          &mqgmo,
          buflen,
          mqmsg,
          &msglen,
          &cc,
          &rc );
  if ( cc != MQCC_OK )
  {
   if ( rc == MQRC_NO_MSG_AVAILABLE ) 
   {
    MsgNo--;
    break;
   }
   sprintf( msg, "MQGET failed - reason code %li\n", rc );
   fputs(   msg, stderr );
   exit(    bad_rc );
  }

  ParameterCount = mqmsg -> ParameterCount;
  mqmsg++;                                       /* Skip over MQCFH */

  /* Extract the Count field i.e. number of balances                */

    if ( ((MQCFIL *)mqmsg) -> Type == MQCFT_INTEGER_LIST )
       Count = ((MQCFIL *)mqmsg) -> Count;
  else Count = ((MQCFSL *)mqmsg) -> Count;                                         

  printf( "Message received - contains %d balances\n", Count );

  /* Establish addressability to sub-structures                     */

  for ( Ptr = (MQCFIL *)mqmsg, i = 0 ;
        i < ParameterCount; 
        i++ )
  {

   switch ( Ptr -> Parameter ) 
   {

   case BALANCES:
    Balances = (MQCFIL *)Ptr;
    break;

   case ACCOUNTHOLDERS:
    AccountHolders = (MQCFSL *)Ptr;
    break;

   case ACCOUNTNUMBERS:
    AccountNumbers = (MQCFSL *)Ptr;
    break;

   default:
    fprintf( stderr, "Corrupt message received - exiting\n" );
    exit( bad_rc );
    
   }   /* switch                      */

  bPtr = (PMQBYTE)Ptr;
  bPtr += Ptr -> StrucLength;
  Ptr = (MQCFIL *)bPtr;

  }    /* establishing addressability */ 

  for ( i = 0, 
        AccountHolderStr = (MQCHAR *)AccountHolders -> Strings,
        AccountNumberStr = (MQCHAR *)AccountNumbers -> Strings ;
        i < Count ; 
        i++,
        AccountHolderStr += ACCOUNTHOLDERLEN,
        AccountNumberStr += ACCOUNTNUMBERLEN )
  {
   printf( " Balance %d\n", i + 1 );
   CharsToCStr( AccountHolder, AccountHolderStr, ACCOUNTHOLDERLEN );
   CharsToCStr( AccountNumber, AccountNumberStr, ACCOUNTNUMBERLEN );
   printf( "   AccountHolder: %s\n", AccountHolder );
   printf( "   AccountNumber: %s\n", AccountNumber );
   printf( "   CurrentBalance: %d\n", Balances -> Values[i] ); 
  }
  printf( "\n" );
 }   /* for loop */

 printf( "%d messages were received - now exiting\n", MsgNo );
 MQCLOSE( hconn,
          &hq,
          MQCO_NONE,
          &cc,
          &rc );
 MQDISC( &hconn, &cc, &rc );
 exit( good_rc );

}  /* main */
