/***********************************************************************/
/*                                                                     */
/* Simple sample program to put a single message of type NBALANCE.     */
/* In this example, NBALANCE is held in PCF format.                    */
/*                                                                     */
/* Copyright IBM Corp 1997.                                            */
/*                                                                     */
/***********************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <cmqc.h>
#include <cmqcfc.h>
#include "NBalance.h"

#define MAX_TEXT 72

enum rc_e { good_rc, bad_rc };

int main( int argc, char *argv[] )
{

 static MQCHAR48   qmname = { '\0'};            /* Queue manager name   */
 MQHCONN           hconn;                       /* Connection handle    */
 MQLONG            cc;                          /* Completion code      */
 MQLONG            rc;                          /* Reason               */
 char              msg[ MAX_TEXT ];             /* Error message        */
 int               i;                           /* Index variable       */
 static MQOD       mqod  = { MQOD_DEFAULT };    /* Object descriptor    */
 static MQMD       mqmd  = { MQMD_DEFAULT };    /* Message descriptor   */
 static MQPMO      mqpmo = { MQPMO_DEFAULT };   /* Put options          */
 int               msg_len;                     /* Message length       */
 MQCFH             *mqmsg;                       /* MQ msg to send       */
 MQCFIL            *Balances;                   /* Balances             */
 MQCFSL            *AccountHolders;             /* Account Holders      */
 MQCFSL            *AccountNumbers;             /* Account Numbers      */
 int               Count;                       /* # of Balance structs */
 int               StrucLength;                 /* Structure Length     */
 PMQCHAR           AccountHolderStr;            /* Working ptr          */
 PMQCHAR           AccountNumberStr;            /* Working ptr          */
 PMQLONG           Balance;                     /* Working ptr          */
 MQCFIL            mqcfil = { MQCFIL_DEFAULT }; /* Default structure    */
 MQCFSL            mqcfsl = { MQCFSL_DEFAULT }; /* Default structure    */
 MQCFH             mqcfh  = { MQCFH_DEFAULT };  /* Default structure    */
 MQBYTE           *bPtr;                        /* For ptr arithmetic   */

 if ( argc !=2 )
 {
  sprintf( msg, "Usage: %s queue_name\n", argv[0] );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }
 strcpy( mqod.ObjectName, argv[ 1 ] );

 MQCONN( qmname, &hconn, &cc, &rc );  /* Connect to default QMgr */
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQCONN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }
 
 strncpy( mqmd.Format, MQFMT_PCF, sizeof( mqmd.Format ) );

 /* Calculate size of message and allocate storage                   */

 Count = 2;                            /* Number of Balances to send */
 msg_len =   MQCFH_STRUC_LENGTH + MQCFIL_STRUC_LENGTH_FIXED + 
           ( 2 * MQCFSL_STRUC_LENGTH_FIXED ) +
           ( Count * ( sizeof( MQLONG ) + ( sizeof( MQCHAR ) * 
             ( ACCOUNTHOLDERLEN + ACCOUNTNUMBERLEN ) ) ) ); 
 mqmsg = calloc( msg_len, sizeof( MQBYTE ) );
 if ( mqmsg == NULL )
 {
  fprintf( stderr, "calloc failed\n" );
  exit(    bad_rc );
 }

 /* Initialise sub-stuctures in message, and establish addressability */
 /* to them. Note that the MQCFH header is mandatory.                 */

 bPtr = (MQBYTE *)mqmsg;
 bPtr += MQCFH_STRUC_LENGTH;
 Balances = (MQCFIL *)(bPtr);
 bPtr += MQCFIL_STRUC_LENGTH_FIXED + ( Count * sizeof( MQLONG ) ); 
 AccountHolders = (MQCFSL *)bPtr;
 bPtr += MQCFSL_STRUC_LENGTH_FIXED + 
         ( Count * ( sizeof( MQCHAR ) * ACCOUNTHOLDERLEN ) );
 AccountNumbers = (MQCFSL *)bPtr;
 memcpy( mqmsg, &mqcfh,  MQCFH_STRUC_LENGTH );
 mqmsg -> ParameterCount = 3;       /* Number of sub-structures      */
 memcpy( Balances, &mqcfil, MQCFIL_STRUC_LENGTH_FIXED ); 
 memcpy( AccountHolders, &mqcfsl, MQCFSL_STRUC_LENGTH_FIXED ); 
 memcpy( AccountNumbers, &mqcfsl, MQCFSL_STRUC_LENGTH_FIXED ); 

 /* Set count field in sub-structures                                */

 Balances -> Count       = Count;
 AccountHolders -> Count = Count;
 AccountNumbers -> Count = Count;
 
 /* The Parameter field identifies the sub-structures to the         */
 /* receiving program, which need not rely on the order in which     */
 /* the sub-structures are assembed in the message.                  */

 Balances -> Parameter       = BALANCES;
 AccountHolders -> Parameter = ACCOUNTHOLDERS;
 AccountNumbers -> Parameter = ACCOUNTNUMBERS;
 
 /* Set StringLength and StrucLengths                                */
 /* Note that StrucLengths must be a multiple of four                */

 AccountHolders -> StringLength = 
                    sizeof( MQCHAR ) * ACCOUNTHOLDERLEN;
 AccountNumbers -> StringLength = 
                    sizeof( MQCHAR ) * ACCOUNTNUMBERLEN;
 Balances -> StrucLength += Count * sizeof( MQLONG );
 StrucLength = MQCFSL_STRUC_LENGTH_FIXED +   
             ( Count * sizeof( MQCHAR ) * ACCOUNTHOLDERLEN );
 StrucLength = 4 * ( ( StrucLength + 3 ) / 4 );
 AccountHolders -> StrucLength = StrucLength;
 StrucLength = MQCFSL_STRUC_LENGTH_FIXED + 
             ( Count * sizeof( MQCHAR ) * ACCOUNTNUMBERLEN );
 StrucLength = 4 * ( ( StrucLength + 3 ) / 4 );
 AccountNumbers -> StrucLength = StrucLength;

 /* Insert data into message                                        */

 Balance = Balances -> Values;
 AccountHolderStr = &( AccountHolders -> Strings[0] );
 AccountNumberStr = &( AccountNumbers -> Strings[0] );

 *Balance = 3000000;
 CStrToChars( AccountHolderStr, "Megabucks Stefan", ACCOUNTHOLDERLEN ); 
 CStrToChars( AccountNumberStr, "123456",           ACCOUNTNUMBERLEN );

 Balance++;
 AccountHolderStr += AccountHolders -> StringLength;
 AccountNumberStr += AccountNumbers -> StringLength;

 *Balance = 2;
 CStrToChars( AccountHolderStr, "Penury Stefan", ACCOUNTHOLDERLEN ); 
 CStrToChars( AccountNumberStr, "123321",        ACCOUNTNUMBERLEN );
  
 MQPUT1( hconn,
        &mqod,
        &mqmd,
        &mqpmo,
         msg_len,
         mqmsg,
        &cc,
        &rc );
  if ( cc != MQCC_OK )
  {
   sprintf( msg, "MQPUT1 failed - reason code %li\n", rc );
   fputs(   msg, stderr );
   exit(    bad_rc );
  }

  MQDISC( &hconn, &cc, &rc );
  exit( good_rc );

}  /* main */
